@@ -164,7 +164,12 @@ def histplot(
 
     hists = list(process_histogram_parts(H, bins))
     final_bins, xtick_labels = get_plottable_protocol_bins(hists[0].axes[0])
-    x_axes_label = get_histogram_axes_title(hists[0].axes[0])
+    _x_axes_label = ax.get_xlabel()
+    x_axes_label = (
+        _x_axes_label
+        if _x_axes_label != ""
+        else get_histogram_axes_title(hists[0].axes[0])
+    )
 
     # TODO: use hists everywhere
     h = np.stack([h.values().astype(float) for h in hists])
@@ -364,7 +369,7 @@ def histplot(
         for i in range(len(hists)):
             _kwargs = _chunked_kwargs[i]
             _f = ax.stairs(h[i], final_bins, label=_labels[i], fill=True, **_kwargs)
-        return_artists.append(StairsArtists(_f, None, None))
+            return_artists.append(StairsArtists(_f, None, None))
         _artist = _f
 
     elif histtype == "errorbar":
@@ -483,9 +488,6 @@ def hist2dplot(
         if not isinstance(ax, plt.Axes):
             raise ValueError("ax must be a matplotlib Axes object")
 
-    x_axes_label = ""
-    y_axes_label = ""
-
     hist = hist_object_handler(H, xbins, ybins)
 
     # TODO: use Histogram everywhere
@@ -495,8 +497,14 @@ def hist2dplot(
     xbin_centers = xbins[1:] - np.diff(xbins) / float(2)
     ybin_centers = ybins[1:] - np.diff(ybins) / float(2)
 
-    x_axes_label = get_histogram_axes_title(hist.axes[0])
-    y_axes_label = get_histogram_axes_title(hist.axes[1])
+    _x_axes_label = ax.get_xlabel()
+    x_axes_label = (
+        _x_axes_label if _x_axes_label != "" else get_histogram_axes_title(hist.axes[0])
+    )
+    _y_axes_label = ax.get_ylabel()
+    y_axes_label = (
+        _y_axes_label if _y_axes_label != "" else get_histogram_axes_title(hist.axes[1])
+    )
 
     H = H.T
 
@@ -541,17 +549,24 @@ def hist2dplot(
 
     plt.sca(ax)
 
-    text_artists = []
-    if labels is not None:
-        if labels is True:
-            _labels = H
-        elif H.shape == labels.T.shape:
-            _labels = labels.T
-        elif labels is not False:
+    _labels: np.ndarray | None = None
+    if isinstance(labels, bool):
+        _labels = H if labels else None
+    elif np.iterable(labels):
+        label_array = np.asarray(labels).T
+        if H.shape == label_array.shape:
+            _labels = label_array
+        else:
             raise ValueError(
-                "Labels not understood, either specify a bool or a Hist-like array"
+                f"Labels input has incorrect shape (expect: {H.shape}, got: {label_array.shape})"
             )
+    elif labels is not None:
+        raise ValueError(
+            "Labels not understood, either specify a bool or a Hist-like array"
+        )
 
+    text_artists = []
+    if _labels is not None:
         for ix, xc in enumerate(xbin_centers):
             for iy, yc in enumerate(ybin_centers):
                 color = (
